#!/usr/bin/env python
# -*- coding: utf-8 -*-

from gimpfu import *
import pygtk
import gtk
from sys import argv, platform
from ctypes import *

####################################################################################################################
################# START gegl_command by Kevin Payne & Claude Lion ##################################################
####################################################################################################################

def load_library (library_name):
    if platform == "linux" or platform == "linux2":
        library_name = library_name + '.so.0'
    elif platform == "win32":
        from ctypes.util import find_library
        library_name = find_library (library_name + "-0")
    else:
        raise BaseException ("TODO")
    return CDLL (library_name)

gimp = load_library ('libgimp-2.0')

def use_gegl_graph(image, drawable, gegl_graph_string):
    class GeglBuffer(Structure):
        pass
    drawable_id = drawable.ID
    
    gegl = load_library ('libgegl-0.4')
    sucess = gegl.gegl_init (None, None)
    gimp.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
    gimp.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

    x = c_int (); y = c_int (); w = c_int (); h = c_int ()
    non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
    args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
    
    if non_empty:
        source = gimp.gimp_drawable_get_buffer (drawable_id)
        target = gimp.gimp_drawable_get_shadow_buffer (drawable_id)
        sucess = gegl.gegl_render_op (source, target, "gegl:gegl", *args)
        gegl.gegl_buffer_flush (target)
        gimp.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
        gimp.gimp_drawable_update (drawable_id, x, y, w, h)
        gimp.gimp_displays_flush ()

####################################################################################################################
################# END gegl_command #################################################################################
####################################################################################################################

def color_line_or_bw_pencil_drawing(image, drawable, bw):
    # put your GEGL command here

    #Start undo
    pdb.gimp_image_undo_group_start(image)
    
    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()

    #Make active the image 
    drawable = pdb.gimp_image_get_active_drawable(image)       
    pdb.gimp_image_set_active_layer(image, drawable)
       
    #Duplicate the image
    #Copy #1
    non_empty = pdb.gimp_edit_copy(drawable)
    copyLayer1 = pdb.gimp_layer_new(image, image.width, image.height, RGBA_IMAGE, "Edit", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(image, copyLayer1, None, 0)
    copyLayer1 = pdb.gimp_edit_paste(copyLayer1, FALSE)
    pdb.gimp_floating_sel_anchor(copyLayer1) 
    copyLayer1 = pdb.gimp_image_get_layer_by_name(image, "Edit") 
    
#################### Create the effect ######################################################
    
    #Apply unsharp mask
    gegl_graph_string="unsharp-mask std-dev=5 scale=3 threshold=0"
    use_gegl_graph(image, copyLayer1, gegl_graph_string)

    #Desaturate
    gegl_graph_string="saturation scale=0"
    use_gegl_graph(image, copyLayer1, gegl_graph_string)
    
    #Black & white pencil
    if bw:
        gegl_graph_string="levels in-low=0.0010   in-high=0.1  out-low=0  out-high=1" 
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
        gegl_graph_string="gamma value=0.1" 
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
        gegl_graph_string="unsharp-mask std-dev=100 scale=1.5 threshold=0"
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
        gegl_graph_string="unsharp-mask std-dev=100 scale=10 threshold=0"
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
   
    #Color pencil
    else:
        gegl_graph_string="levels in-low=0.0010   in-high=0.1  out-low=0  out-high=1" 
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
        gegl_graph_string="gamma value=0.1" 
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
        gegl_graph_string="unsharp-mask std-dev=100 scale=1.5 threshold=0"
        use_gegl_graph(image, copyLayer1, gegl_graph_string)
        pdb.gimp_layer_set_mode(copyLayer1, LAYER_MODE_HSV_VALUE)
        copyLayer2 = pdb.gimp_layer_new_from_drawable(drawable, image)
        pdb.gimp_image_insert_layer(image, copyLayer2, None, 0)
        pdb.gimp_layer_set_mode(copyLayer2, LAYER_MODE_ADDITION)
        pdb.gimp_layer_set_opacity(copyLayer2, 100)
    
    pdb.gimp_displays_flush(image)

    #End undo
    pdb.gimp_image_undo_group_end(image)
    
    #Set gimp to default
    pdb.gimp_context_pop()
        
register(
    "color_line_or_bw_pencil_drawing",
    "Color line pencil or black & white drawing",
    "Transform Your Image Into color line or black & white pencil drawing",
    "Pocholo",
    "Poholo",
    "2022",
    "Color line or BW pencil drawing",
    "",
    [
    (PF_IMAGE,      "image",       "Input image", None),
    (PF_DRAWABLE,   "drawable", "Input drawable", None),
    (PF_TOGGLE, "bw", "B&W Pencil?:", 0),
    ],
    [],
    color_line_or_bw_pencil_drawing, menu="<Image>/Pocholo's-Plugins/"
    )
main()